package com.zopim.android.sdk.api;

import android.support.annotation.NonNull;
import com.zopim.android.sdk.model.ChatLog;
import java.io.File;

/**
 * Chat APIs interface
 * <p>
 * Defines the set of APIs available in a given chat session.
 * APIs will be linked to the account configured during initialization of {@link ZopimChatApi#init(String)}
 * </p>
 */
public interface ChatApiCommands {

    /**
     * Sends the visitor message.
     * <p>
     * This message will appear in the agent dashboard in an active chat queue.
     * </p>
     *
     * @param message to be sent
     * @see <a href="https://dashboard.zopim.com/#visitor_list/">Agent Visitor List</a>
     */
    void send(String message);

    /**
     * Sends the file to the current chat log. Successful file send will trigger a chat log update with information
     * about the file being uploaded.
     * <p>
     * File must pass validation check and upload requirements defined in
     * <a href="https://zopim.zendesk.com/entries/23268514-Sending-and-receiving-files-on-Zopim">Zopim attachments page</a>
     * </p>
     * <p>
     * A successful file upload will be visible in the agent dashboard.
     * </p>
     *
     * @param file to send
     * @see com.zopim.android.sdk.data.observers.ChatLogObserver
     * @see <a href="https://dashboard.zopim.com">Agent Dashboard</a>
     */
    void send(@NonNull File file);

    /**
     * Re-sends the visitor message.
     * <p>
     * This message will appear in the agent dashboard.
     * </p>
     *
     * @param messageId id of the message to resend
     * @see <a href="https://dashboard.zopim.com/#visitor_list/">Agent Visitor List</a>
     */
    void resend(String messageId);

    /**
     * Sends email to agents defined in the dashboard for this account.
     * <p>
     * Email must be provided in order for the agent to be able to respond.
     * </p>
     *
     * @param name    of the visitor
     * @param email   of the visitor
     * @param message inquiry
     * @return true if was able to send, false otherwise
     */
    boolean sendOfflineMessage(String name, String email, String message);

    /**
     * Sets rating for the current chat session.
     * <p>
     * This will appear as a response to the chat rating questionnaire sent by an agent to the visitor.
     * </p>
     *
     * @param rating to set
     */
    void sendChatRating(@NonNull ChatLog.Rating rating);

    /**
     * Sets a comment to the current chat session.
     *
     * @param comment to set
     */
    void sendChatComment(@NonNull String comment);

    /**
     * Emails the current chat transcript
     * <p>
     * It will send an email to the provided address with the current chat log
     * </p>
     *
     * @param email address to send the transcript to
     * @return true if email was sent, false otherwise
     */
    boolean emailTranscript(String email);

    /**
     * Ends the current chat. This will update the connection status to closed.
     */
    void endChat();

    /**
     * Disconnect the current chat, used when switching to push mode.
     */
    void disconnect();

    /**
     * Sets a department to the current chat session.
     * <p>
     * Configuring a department in the chat will deliver the chat to the right group of agents supporting that
     * department's inquiries.
     * This information will be visible in the agent dashboard.
     * </p>
     * <p>
     * Provided department should be one of the departments available in this account.
     * The list of available departments will be available in
     * {@link com.zopim.android.sdk.data.LivechatDepartmentsPath} after initialization is complete {@link ChatService#onChatInitialized()}.
     * Setting a non defined department will have no effect.
     * </p>
     *
     * @param department visitor is chatting with
     * @see <a href="https://dashboard.zopim.com/#departments">Departments Dashboard</a>
     */
    void setDepartment(String department);

    /**
     * Sets phone number information for the user that is in the current chat session.
     * <p>
     * This information will be visible in the agent dashboard.
     * </p>
     *
     * @param phoneNumber of the visitor
     * @see <a href="https://dashboard.zopim.com">Agent Dashboard</a>
     */
    void setPhoneNumber(String phoneNumber);

    /**
     * Sets the name for the user in the current chat session.
     * <p>
     * This information will be visible in the agent dashboard.
     * </p>
     *
     * @param name of the visitor
     * @see <a href="https://dashboard.zopim.com">Agent Dashboard</a>
     */
    void setName(String name);

    /**
     * Sets the email for the user in the current chat session.
     * <p>
     * This information will be visible in the agent dashboard.
     * </p>
     *
     * @param email of the visitor
     * @see <a href="https://dashboard.zopim.com">Agent Dashboard</a>
     */
    void setEmail(String email);

    /**
     * Sets a note to the visitor profile
     *
     * @param note about the visitor
     */
    void setNote(String note);

    /**
     * Appends a note to the current visitor note
     *
     * @param note about the visitor to append
     */
    void addNote(String note);
}
