package com.zopim.android.sdk.data;

import android.support.annotation.Nullable;
import com.zopim.android.sdk.api.ObservableTrigger;
import com.zopim.android.sdk.data.observers.AccountObserver;
import com.zopim.android.sdk.data.observers.AgentsObserver;
import com.zopim.android.sdk.data.observers.ChatLogObserver;
import com.zopim.android.sdk.data.observers.ConnectionObserver;
import com.zopim.android.sdk.data.observers.DepartmentsObserver;
import com.zopim.android.sdk.data.observers.FileSendingObserver;
import com.zopim.android.sdk.data.observers.FormsObserver;
import com.zopim.android.sdk.data.observers.ProfileObserver;
import com.zopim.android.sdk.model.Account;
import com.zopim.android.sdk.model.Agent;
import com.zopim.android.sdk.model.ChatLog;
import com.zopim.android.sdk.model.Connection;
import com.zopim.android.sdk.model.Department;
import com.zopim.android.sdk.model.FileSending;
import com.zopim.android.sdk.model.Forms;
import com.zopim.android.sdk.model.Profile;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Interface definition of chat data source.
 * It allows data retrieval and subscription to data change broadcasts.
 */
public interface DataSource {

    /**
     * Removes all observers from all data paths
     */
    void deleteObservers();

    /**
     * Clear all data source.
     * <p>
     * This will clear each data path
     * </p>
     */
    void clear();

    /**
     * Gets connection object from the {@link com.zopim.android.sdk.data.ConnectionPath}
     *
     * @return connection pojo
     * @see ConnectionPath#getData()
     * @see com.zopim.android.sdk.model.Connection
     */
    Connection getConnection();

    /**
     * Gets profile object from the {@link com.zopim.android.sdk.data.LivechatProfilePath}
     *
     * @return profile pojo
     * @see LivechatProfilePath#getData()
     * @see com.zopim.android.sdk.model.Profile
     */
    Profile getProfile();

    /**
     * Gets agents in the current chat. Agents are defined in {@link com.zopim.android.sdk.data.LivechatAgentsPath}
     *
     * @return agents and their ID
     * @see LivechatAgentsPath#getData()
     * @see com.zopim.android.sdk.model.Agent
     */
    LinkedHashMap<String, Agent> getAgents();

    /**
     * Gets departments from the {@link com.zopim.android.sdk.data.LivechatDepartmentsPath}
     *
     * @return departments and their ID
     * @see com.zopim.android.sdk.data.LivechatDepartmentsPath#getData()
     * @see com.zopim.android.sdk.model.Department
     */
    Map<String, Department> getDepartments();

    /**
     * Gets account object from the {@link com.zopim.android.sdk.data.LivechatAccountPath}
     *
     * @return account pojo
     * @see com.zopim.android.sdk.data.LivechatAccountPath#getData()
     * @see com.zopim.android.sdk.model.Account
     */
    Account getAccount();

    /**
     * Gets messages from the {@link LivechatChatLogPath}
     *
     * @return messages and their ID
     * @see LivechatChatLogPath#getData()
     * @see com.zopim.android.sdk.model.ChatLog
     */
    LinkedHashMap<String, ChatLog> getChatLog();

    /**
     * Gets forms from the {@link com.zopim.android.sdk.data.LivechatFormsPath}
     *
     * @return forms model
     * @see LivechatFormsPath#getData()
     * @see com.zopim.android.sdk.model.Forms
     */
    Forms getForms();

    /**
     * Gets file sending model from the {@link com.zopim.android.sdk.data.LivechatFileSendingPath}
     *
     * @return file sending model
     * @see LivechatFileSendingPath#getData()
     * @see com.zopim.android.sdk.model.FileSending
     */
    @Nullable
    FileSending getFileSending();

    /**
     * Adds observer to the {@link com.zopim.android.sdk.data.ConnectionPath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.Connection}.
     * You can register {@link com.zopim.android.sdk.data.observers.ConnectionObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addConnectionObserver(ConnectionObserver observer);

    /**
     * Adds observer to the {@link com.zopim.android.sdk.data.LivechatProfilePath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.Profile}.
     * You can register {@link com.zopim.android.sdk.data.observers.ProfileObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addProfileObserver(ProfileObserver observer);

    /**
     * Adds observer to the {@link com.zopim.android.sdk.data.LivechatAgentsPath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.Agent}.
     * You can register {@link com.zopim.android.sdk.data.observers.AgentsObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addAgentsObserver(AgentsObserver observer);

    /**
     * Adds observer to the {@link com.zopim.android.sdk.data.LivechatDepartmentsPath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.Department}.
     * You can register {@link com.zopim.android.sdk.data.observers.DepartmentsObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addDepartmentsObserver(DepartmentsObserver observer);

    /**
     * Adds observer to the {@link com.zopim.android.sdk.data.LivechatAccountPath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.Account}.
     * You can register {@link com.zopim.android.sdk.data.observers.AccountObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addAccountObserver(AccountObserver observer);

    /**
     * Adds observer to the {@link com.zopim.android.sdk.data.LivechatFormsPath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.Forms}.
     * You can register {@link com.zopim.android.sdk.data.observers.FormsObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addFormsObserver(FormsObserver observer);

    /**
     * Adds observer to the {@link com.zopim.android.sdk.data.LivechatFileSendingPath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.FileSending}.
     * You can register {@link com.zopim.android.sdk.data.observers.FileSendingObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addFileSendingObserver(FileSendingObserver observer);

    /**
     * Adds observer to the {@link LivechatChatLogPath}
     * <p>
     * Observer will be registered for broadcast updates on {@link com.zopim.android.sdk.model.ChatLog}.
     * You can register {@link com.zopim.android.sdk.data.observers.ChatLogObserver} or {@link java.util.Observer}.
     * </p>
     *
     * @param observer to register
     * @return trigger for requesting an update
     */
    ObservableTrigger addChatLogObserver(ChatLogObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.ConnectionPath}
     *
     * @param observer to remove
     */
    void deleteConnectionObserver(ConnectionObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.LivechatProfilePath}
     *
     * @param observer to remove
     */
    void deleteProfileObserver(ProfileObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.LivechatAgentsPath}
     *
     * @param observer to remove
     */
    void deleteAgentsObserver(AgentsObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.LivechatAccountPath}
     *
     * @param observer to remove
     */
    void deleteAccountObserver(AccountObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.LivechatDepartmentsPath}
     *
     * @param observer to remove
     */
    void deleteDepartmentsObserver(DepartmentsObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.LivechatDepartmentsPath}
     *
     * @param observer to remove
     */
    void deleteChatLogObserver(ChatLogObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.LivechatFormsPath}
     *
     * @param observer to remove
     */
    void deleteFormsObserver(FormsObserver observer);

    /**
     * Deletes observer from the {@link com.zopim.android.sdk.data.LivechatFileSendingPath}
     *
     * @param observer to remove
     */
    void deleteFileSendingObserver(FileSendingObserver observer);
}
