package com.zopim.android.sdk.util;

import java.util.Collection;
import java.util.Queue;

/**
 * Definition of interface queue that maintains the queue size by removing elements from the head of the queue if the capacity is violated. This queue orders elements FIFO (first-in-first-out).
 */
public interface CircularQueue<E> {

    /**
     * Inserts the specified element into this queue if it is possible to do so immediately without violating capacity restrictions.
     * <p>
     * Violating capacity restrictions will result in a removal of an element at the head of the queue to accommodate space in the queue for a newly added element.
     * </p>
     *
     * @param element to add
     */
    void offer(E element);

    /**
     * Retrieves and removes the head of this queue, or returns null if this queue is empty.
     *
     * @return the head of this queue, or null if this queue is empty
     * @see Queue#poll()
     */
    E poll();

    /**
     * Appends all of the elements in the specified collection to the end of this queue, in the order that they are returned by the specified collection's iterator.
     * Attempts to addAll of a queue to itself result in IllegalArgumentException.
     *
     * @param collection the elements to be inserted into this queue
     * @see Queue#addAll(Collection)
     */
    void addAll(Collection<? extends E> collection);

    /**
     * Returns true if this queue contains no elements.
     *
     * @return true if this queue contains no elements
     * @see Queue#isEmpty()
     */
    boolean isEmpty();

    /**
     * Returns the number of elements in this queue.
     *
     * @return the number of elements in this queue
     * @see Queue#size()
     */
    int size();

    /**
     * Removes all of the elements from this queue. The queue will be empty after this call returns.
     *
     * @see Queue#clear()
     */
    void clear();

    /**
     * Returns an array containing all of the elements in this queue, in proper sequence; the runtime type of the returned array is that of the specified array. If the queue fits in the specified array, it is returned therein. Otherwise, a new array is allocated with the runtime type of the specified array and the size of this queue.
     *
     * @param a The array into which the elements of the queue are to be stored.
     * @return an array of the elements
     * @see Queue#toArray(Object[])
     */
    E[] toArray(E[] a);

    /**
     * Returns true if this queue contains the specified element. More formally, returns true if and only if this queue contains at least one element e such that (o==null ? e==null : o.equals(e)).
     *
     * @param element to look for in the queue
     * @return true if this queue contains the specified element
     */
    boolean contains(E element);
}
